#include <SPI.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>

#include <Adafruit_PCD8544.h>
#define PCF8574_I2C_Address 0x27     // Define the I2C address

// ==============================================================================================================
//                                              Define Pins
// ==============================================================================================================

// Pins below are used by the Counter PCB and can not be changed
#define Ready 3                                                       // Define Ready Pin
#define Trigger 2                                                     // Define Trig Pin
#define RFInput 4                                                     // Define RF input Pin

// ==============================================================================================================
//                                        Define LCD display 2x16 Char
// ==============================================================================================================

LiquidCrystal_I2C lcd(0x27, 16, 2);                                   // LCD address to 0x27 

// ==============================================================================================================
//                                           Define Variables
// ==============================================================================================================

long Incomming_Data=0;                                                // Counter Value

// ==============================================================================================================
//                                              Functions
// ==============================================================================================================
// The function  "GetCounterValue" count the frequency from selected input.
// The RF input is choosen by passing 0 or 1 to the function.  0=Low-Input, and 1=High-Input
// Exmaple : GetCounterValue(0) will return the counting of Low input
// Exmaple : GetCounterValue(1) will return the counting of High input

    long GetCounterValue(byte SignalInput){
    
	  unsigned long startedWaiting=0;                                   // Used for timeout
	  long CounterValue=0;                                              // Value from Counter

    // Check which input to connect to the counter (50 MHz input or 2.5 GHz)		
	  if(SignalInput==0) {
        digitalWrite(RFInput,LOW);                                    // Set Pin Low to select 50MHz input
	  }
        else{
        digitalWrite(RFInput,HIGH);                                   // Set Pin Low to select 2.5GHz input
	  }
    delay(50);
    // Send a trig pulse High-Low to start counting
    digitalWrite(Trigger,HIGH);                                       // Trigger goes High
    delayMicroseconds(100);
    digitalWrite(Trigger,LOW);                                        // Trigger goes Low
    
    delayMicroseconds(100);

    // Wait for the counter to reply
    // If no reply comes within 1.5 sec something is wrong and jump out 
    startedWaiting = millis();
    while(digitalRead(Ready)==0 && millis()-startedWaiting<=1500){
    }	
	  // Counter has replied and Counting value can be transfered to arduino
	  // Read all the bits to form the Counter value
    for (int Bitcounter = 1; Bitcounter <= 32; Bitcounter++) {
        digitalWrite(Trigger,HIGH);                                   // Trigger (Clock) goes High	
        delayMicroseconds(100);
		    CounterValue=CounterValue<<1;                                 // Shift Counter value one bit left
        if(digitalRead(Ready)==1){
            bitSet(CounterValue,0);                                   // If pin is high, set bit High 
        }
        digitalWrite(Trigger,LOW);                                    // Trigger (Clock) goes Low	
        delayMicroseconds(100);
    }  
    return CounterValue;                                              // Send back the counter Value
}	
// ---------------------------------------------------------------------------------------------------------- END




// ==============================================================================================================
//                                              SETUP
// ==============================================================================================================

void setup(){
    Serial.begin(9600);                                               // Init serial
    Wire.begin();                                                     // Init I2c

    pinMode(Trigger, OUTPUT);                                         // Set Pin as Output
    digitalWrite(Trigger,LOW);                                        // Set Pin Low     
    pinMode(RFInput, OUTPUT);                                         // Set Pin as Output 
    digitalWrite(RFInput,LOW);                                        // Set Pin Low
    pinMode(Ready, INPUT);                                            // Set pin as Input
    delay(500);                                                       // Shield to wake up

// LCD Initiering ----------------------------------------------------------------------------------------- START
    lcd.backlight();
    //lcd.noBacklight();

    lcd.init();                                                       // initialize the lcd 
// LCD Initiering ------------------------------------------------------------------------------------------- END

}



// ==============================================================================================================
//                                              Loop
// ==============================================================================================================
void loop() {

    Incomming_Data=GetCounterValue(0);                                // Get Counter Value Low input
    //Incomming_Data=GetCounterValue(1);                              // Get Counter Value High input

    lcd.clear();                                                      // Clear display   
    lcd.setCursor(0, 0);                                              // Position  
    lcd.print("Frequency Input");                                     // Print Value  
    lcd.setCursor(0, 1);                                              // Position        
    lcd.print(Incomming_Data);                                        // Print Value  

    
    Serial.println(Incomming_Data,DEC);                               // Print out counter value
    
    }    
